# Generated by Django 3.1.3 on 2020-11-05 15:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('vendor', '0007_offer_list_bundle_items'),
    ]

    operations = [
        migrations.AddField(
            model_name='offer',
            name='allow_multiple',
            field=models.BooleanField(default=False, help_text='Confirm the user wants to buy multiples of the product where typically there is just one purchased at a time.', verbose_name='Allow Multiple Purchase'),
        ),
    ]



from django.core.exceptions import ObjectDoesNotExist
from integrations.models import Credential
from vendor.forms import AuthorizeNetIntegrationForm


# TODO: Could implement an IntegrationBase Class in package.
class AuthorizeNetIntegration(object):
    NAME = "AuthorizeNet Integration"

    site = None
    form_class = AuthorizeNetIntegrationForm

    def __init__(self, site):
        self.site = site
        self.instance = self.get_instance()

    def get_instance(self):
        try:
            return Credential.objects.get(name=self.NAME, site=self.site)
        except ObjectDoesNotExist:
            return None
    
    def save(self, data):
        if not self.instance:
            form = self.form_class(data)
        else:
            form = self.form_class(data, instance=self.instance)
        
        authorizenet = form.save(commit=False)
        authorizenet.name = self.NAME
        authorizenet.site = self.site
        authorizenet.save()


# Generated by Django 3.1 on 2020-10-28 17:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('vendor', '0006_auto_20201005_2257'),
    ]

    operations = [
        migrations.AddField(
            model_name='offer',
            name='list_bundle_items',
            field=models.BooleanField(default=False, help_text='When showing to customers, display the included items in a list?', verbose_name='List Bundled Items'),
        ),
    ]



import json
import hashlib
import hmac
import logging

from django.conf import settings
from django.core.exceptions import PermissionDenied
from django.http import JsonResponse
from django.views import View
from django.views.decorators.csrf import csrf_exempt
from django.utils import timezone

from vendor.models import Receipt, Invoice
from vendor.processors.authorizenet import AuthorizeNetProcessor

logger = logging.getLogger(__name__)

payment_processor = AuthorizeNetProcessor


def renew_subscription_task(json_data):
    """
    function to be added or called to a task queue to handle the the a subscription renewal.
    """
    transaction_id = json_data['payload']['id']
    dummy_invoice = Invoice()

    processor = payment_processor(dummy_invoice)
    logger.info(f"Getting transaction detail for id: {transaction_id}")
    transaction_detail = processor.get_transaction_detail(transaction_id)

    if not hasattr(transaction_detail, 'subscription'):
        return None
    if transaction_detail.subscription.payNum.pyval == 1:
        return None

    past_receipt = Receipt.objects.filter(transaction=transaction_detail.subscription.id.text).order_by('created').last()

    payment_info = {
        'account_number': transaction_detail.payment.creditCard.cardNumber.text[-4:],
        'account_type': transaction_detail.payment.creditCard.cardType.text,
        'full_name': " ".join([transaction_detail.billTo.firstName.text, transaction_detail.billTo.lastName.text]),
        'raw': str(json_data),
        'transaction_id': transaction_id,
        'subscription_id': transaction_detail.subscription.id.text,
        'payment_number': transaction_detail.subscription.payNum.text
    }

    invoice_history = []

    for receipt in Receipt.objects.filter(transaction=transaction_detail.subscription.id.text).order_by('created'):
        past_invoice = receipt.order_item.invoice
        invoice_info = {
            "invoice_id": past_invoice.pk,
            "receipt_id": receipt.pk,
            "payments": [payment.pk for payment in past_invoice.payments.all()]
        }
        invoice_history.append(invoice_info)

    invoice = Invoice(status=Invoice.InvoiceStatus.PROCESSING, site=past_receipt.order_item.invoice.site)
    invoice.profile = past_receipt.profile
    invoice.ordered_date = timezone.now()
    invoice.vendor_notes['history'] = invoice_history
    invoice.save()
    invoice.add_offer(past_receipt.order_item.offer)
    invoice.total = transaction_detail.authAmount.pyval
    invoice.save()

    processor = AuthorizeNetProcessor(invoice)
    processor.renew_subscription(past_receipt, payment_info)


class AuthorizeNetBaseAPI(View):
    """
    Base class to handel Authroize.Net webhooks.
    """

    @csrf_exempt
    def dispatch(self, *args, **kwargs):
        """
        Dispatch override to accept post entries without csfr given that they include
        a X-Anet-Signature in there header that must be encoded with Signature Key using
        HMAC-sha512 on the payload.
        Reference: https://developer.authorize.net/api/reference/features/webhooks.html
        """
        return super().dispatch(*args, **kwargs)

    def is_valid_post(self):
        logger.info(f"Request body: {self.request.body}")
        logger.info(f"X ANET SIGNATURE: {self.request.META.get('HTTP_X_ANET_SIGNATURE')}")

        if 'HTTP_X_ANET_SIGNATURE' not in self.request.META:
            logger.warning("Webhook warning Signature KEY")
            return False

        hash_value = hmac.new(bytes(settings.AUTHORIZE_NET_SIGNATURE_KEY, 'utf-8'), self.request.body, hashlib.sha512).hexdigest()
        logger.info(f"Checking hashs\nCALCULATED: {hash_value}\nREQUEST VALUE: {self.request.META.get('HTTP_X_ANET_SIGNATURE')[7:]}")
        if hash_value.upper() == self.request.META.get('HTTP_X_ANET_SIGNATURE')[7:]:
            return True

        return False


class AuthorizeCaptureAPI(AuthorizeNetBaseAPI):
    """
    API endpoint to get event notifications from authorizenet when a authcaputre is created.
    If there is a subscription tied to the transaction, it will renew such subscription
    """

    def post(self, request, *args, **kwargs):
        logger.info(f"AuthorizeNet AuthCapture Event webhook: {request.POST}")

        if not request.body:
            logger.warning("Webhook event has no body")
            return JsonResponse({"msg": "Invalid request body."})

        if not self.is_valid_post():
            logger.error(f"authcapture: Request was denied: {request}")
            # raise PermissionDenied() # This will be uncommented out until we know that a valid call is being properly validated. 

        request_data = json.loads(request.body)
        logger.info(f"Renewing subscription request body: {request_data}")
        renew_subscription_task(request_data)
        logger.info("authcapture subscription renewed")

        return JsonResponse({"msg": "subscription renewed"})



import csv

from itertools import chain
from django.contrib import messages
from django.contrib.sites.models import Site
from django.http import StreamingHttpResponse
from django.shortcuts import redirect
from django.utils.translation import gettext_lazy as _
from django.views.generic.list import BaseListView
from django.views.generic.edit import FormMixin

from vendor.forms import DateRangeForm
from vendor.models import Receipt, Invoice
from vendor.utils import get_site_from_request


class Echo:
    """An object that implements just the write method of the file-like
    interface.
    """

    def write(self, value):
        """Write the value by returning it, instead of storing in a buffer."""
        return value


class CSVStreamRowView(BaseListView):
    """A base view for displaying a list of objects."""

    filename = "receipt_list.csv"

    def get_queryset(self):
        return super().get_queryset()

    def get_row_data(self):
        header = [["ROW_NAME", "ROW_COUNT"]]  # Has to be a list inside an iterable (another list) for the chain to work.
        rows = (["Row {}".format(idx), str(idx)] for idx in range(500))     # Dummy data to show that its working.
        return chain(header, rows)

    def get(self, request, *args, **kwargs):
        rows = self.get_row_data()
        pseudo_buffer = Echo()
        writer = csv.writer(pseudo_buffer)
        # TODO: Figure out what to prepend the header without breaking the streaming generator
        response = StreamingHttpResponse((writer.writerow(row) for row in rows), content_type="text/csv")

        # Set the filename
        response['Content-Disposition'] = 'attachment; filename="{}"'.format(self.filename)
        return response


class ReceiptListCSV(FormMixin, CSVStreamRowView):
    filename = "receipts.csv"
    model = Receipt
    form_class = DateRangeForm
    header = [[_('Order ID'), _('Title'), _('Order Date'), _('Order Status'), _('Order Total'), _('Product ID'), _('Product Name'), _('Quantity'), _('Item Cost'), _('Item Total'), _('Discount Amount '), _('Coupon Code'), _('Coupons Used'), _('Total Discount Amount'), _('Refund ID'), _('Refund Total'), _('Refund Amounts'), _('Refund Reason'), _('Refund Date'), _('Refund Author Email'), _('Date Type'), _('Dates')]]

    def get_queryset(self):
        form = self.form_class(data=self.request.POST)
        start_date = form.data.get('start_date', None)
        end_date = form.data.get('end_date', None)
        if start_date and end_date:
            return self.model.objects.filter(profile__site=Site.objects.get_current(), order_item__invoice__created__gte=start_date, order_item__invoice__created__lte=end_date)
        elif start_date and not end_date:
            return self.model.objects.filter(profile__site=Site.objects.get_current(), order_item__invoice__created__gte=start_date)
        else:
            return self.model.objects.filter(profile__site=Site.objects.get_current())      # Return receipts only for profiles on this site

    def get_row_data(self):
        object_list = self.get_queryset()
        rows = [[
            str(obj.order_item.invoice.pk),              # Order ID
            obj.order_item.invoice,                      # Title
            obj.order_item.invoice.created.isoformat(),  # Oder Date
            obj.get_status_display(),                    # Order Status
            obj.order_item.invoice.total,                # Order Total
            str(obj.products.first().id),                # Product ID
            obj.products.first().name,                   # Product Name
            obj.order_item.quantity,                     # Quantity
            obj.order_item.price,                        # Item Cost
            obj.order_item.total,                        # Item Total
            "",                                          # TODO: Discount Amount
            "",                                          # TODO: Coupon Code
            "",                                          # TODO: Coupons Used
            "",                                          # TODO: Total Discount Amount
            "",                                          # TODO: Refund ID
            "",                                          # TODO: Refund Total
            "",                                          # TODO: Refound Amounts
            "",                                          # TODO: Refund Reason
            "",                                          # TODO: Refund Date
            "",                                          # TODO: Refund Author Email
            'multiple' if obj.end_date is None else f'range',                                          # TODO: Date Type
            " ".join([ '' if obj.start_date is None else f'{obj.start_date:%Y-%m-%d}', '' if obj.end_date is None else f'{obj.end_date:%Y-%m-%d}']),
            ] for obj in object_list]
        return chain(self.header, rows)

    def post(self, request, *args, **kwargs):
        form = self.form_class(request.POST)

        if not form.is_valid():
            messages.info(self.request, ",".join([error for error in form.errors]))
            return redirect(request.META.get('HTTP_REFERER', self.success_url))

        rows = self.get_row_data()
        pseudo_buffer = Echo()
        writer = csv.writer(pseudo_buffer)
        # TODO: Figure out what to prepend the header without breaking the streaming generator
        response = StreamingHttpResponse((writer.writerow(row) for row in rows), content_type="text/csv")

        # Set the filename
        response['Content-Disposition'] = 'attachment; filename="{}"'.format(self.filename)
        return response


class InvoiceListCSV(CSVStreamRowView):
    filename = "invoices.csv"
    model = Invoice

    def get_queryset(self):
        # TODO: Update to handle ranges from a POST
        if hasattr(self.request, 'site'):
            return self.model.objects.filter(site=get_site_from_request(self.request))
        return self.model.on_site.all()

    def get_row_data(self):
        object_list = self.get_queryset()
        header = [["INVOICE_ID", "CREATED_TIME(ISO)", "USERNAME", "CURRENCY", "TOTAL"]]  # Has to be a list inside an iterable (another list) for the chain to work.
        rows = ([str(obj.pk), obj.created.isoformat(), str(obj.profile.user.username), obj.currency, obj.total] for obj in object_list)
        return chain(header, rows)



# Generated by Django 3.1.3 on 2020-11-20 17:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('vendor', '0010_auto_20201112_0138'),
    ]

    operations = [
        migrations.AlterField(
            model_name='address',
            name='address_1',
            field=models.CharField(max_length=40, verbose_name='Address'),
        ),
        migrations.AlterField(
            model_name='address',
            name='address_2',
            field=models.CharField(blank=True, max_length=40, null=True, verbose_name='Address 2 (Optional)'),
        ),
        migrations.AlterField(
            model_name='address',
            name='country',
            field=models.IntegerField(choices=[(581, 'United States')], default=581, verbose_name='Country/Region'),
        ),
    ]



from django.apps import apps
from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.sites.models import Site
from django.db.models import Count, Q
from django.shortcuts import redirect, render, get_object_or_404
from django.urls import reverse, reverse_lazy
from django.views.generic import View
from django.views.generic.detail import DetailView
from django.views.generic.edit import CreateView, UpdateView, FormView
from django.views.generic.list import ListView
from django.utils.translation import gettext as _

from vendor.config import VENDOR_PRODUCT_MODEL, PaymentProcessorSiteConfig, PaymentProcessorSiteSelectSiteConfig, PaymentProcessorForm, PaymentProcessorSiteSelectForm
from vendor.forms import OfferForm, PriceFormSet, CreditCardForm, AddressForm, AuthorizeNetIntegrationForm
from vendor.integrations import AuthorizeNetIntegration
from vendor.models import Invoice, Offer, Receipt, CustomerProfile, Payment
from vendor.models.choice import TermType, PaymentTypes
from vendor.views.mixin import PassRequestToFormKwargsMixin, SiteOnRequestFilterMixin, TableFilterMixin, get_site_from_request
from vendor.processors import get_site_payment_processor

from siteconfigs.models import SiteConfigModel

Product = apps.get_model(VENDOR_PRODUCT_MODEL)
#############
# Admin Views
class AdminDashboardView(LoginRequiredMixin, SiteOnRequestFilterMixin, ListView):
    '''
    List of the most recent invoices generated on the current site.
    '''
    template_name = "vendor/manage/dashboard.html"
    model = Invoice

    def get_queryset(self):
        """
        Return the most recent 10
        """
        queryset = super().get_queryset()

        return queryset[:10]


class AdminInvoiceListView(LoginRequiredMixin, SiteOnRequestFilterMixin, ListView):
    '''
    List of all the invoices generated on the current site.
    '''
    template_name = "vendor/manage/invoice_list.html"
    model = Invoice

    def get_queryset(self):
        """
        Ignores Cart state invoices
        """
        queryset = super().get_queryset()
        return queryset.order_by('updated')


class AdminInvoiceDetailView(LoginRequiredMixin, DetailView):
    '''
    Details of an invoice generated on the current site.
    '''
    template_name = "vendor/manage/invoice_detail.html"
    model = Invoice
    slug_field = 'uuid'
    slug_url_kwarg = 'uuid'


class AdminProductListView(LoginRequiredMixin, TableFilterMixin, SiteOnRequestFilterMixin, ListView):
    '''
    Creates a Product to be added to offers
    '''
    template_name = "vendor/manage/products.html"
    model = Product
    paginate_by = 25

    def search_filter(self, queryset):
        return queryset.filter(name__icontains=self.request.GET.get('search_filter'))


class AdminProductUpdateView(LoginRequiredMixin, UpdateView):
    '''
    Details of an invoice generated on the current site.
    '''
    template_name = "vendor/manage/product.html"
    model = Product
    fields = ['sku', 'name', 'site', 'available', 'description', 'meta']
    slug_field = 'uuid'
    slug_url_kwarg = 'uuid'
    success_url = reverse_lazy('vendor_admin:manager-product-list')


class AdminProductCreateView(LoginRequiredMixin, CreateView):
    '''
    Creates a Product to be added to offers
    '''
    template_name = "vendor/manage/product.html"
    model = Product
    fields = ['sku', 'name', 'site', 'available', 'description', 'meta']
    success_url = reverse_lazy('vendor_admin:manager-product-list')

    def form_valid(self, form):
        new_product = form.save(commit=False)
        new_product.save()
        return redirect(self.success_url)


class AdminOfferListView(LoginRequiredMixin, SiteOnRequestFilterMixin, ListView):
    '''
    Creates a Product to be added to offers
    '''
    template_name = "vendor/manage/offers.html"
    model = Offer


class AdminOfferUpdateView(LoginRequiredMixin, PassRequestToFormKwargsMixin, UpdateView):
    '''
    Details of an invoice generated on the current site.
    '''
    template_name = "vendor/manage/offer.html"
    model = Offer
    form_class = OfferForm
    slug_field = 'uuid'
    slug_url_kwarg = 'uuid'
    template_name_suffix = '_update_form'

    def get_initial(self):
        return {'products': self.object.products.all()}

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        offer_products = self.object.products.all()
        customers_who_own = CustomerProfile.objects.filter(
            receipts__products__in=offer_products)
        customers_who_dont_own = CustomerProfile.objects.all().exclude(
            pk__in=[customer_profile.pk for customer_profile in customers_who_own.all()])

        context['customers_who_own'] = customers_who_own
        context['customers_who_dont_own'] = customers_who_dont_own

        context['formset'] = PriceFormSet(instance=self.object)

        return context

    def form_valid(self, form):
        price_formset = PriceFormSet(
            self.request.POST, self.request.FILES, instance=Offer.objects.get(uuid=self.kwargs['uuid']))

        offer = form.save(commit=False)

        if len(form.cleaned_data['products']) > 1:
            offer.bundle = True
        offer.save()

        for product in form.cleaned_data['products']:
            offer.products.add(product)

        if price_formset.has_changed() and not price_formset.is_valid():
            return render(self.request, self.template_name, {'form': form, 'formset': price_formset})
        elif price_formset.is_valid():
            for price_form in price_formset:
                price = price_form.save(commit=False)
                price.offer = offer
                if price_form.cleaned_data['price_select'] == 'free':
                    price.cost = 0
                price.save()
        else:
            return render(self.request, self.template_name, {'form': form, 'formset': price_formset})

        return redirect('vendor_admin:manager-offer-list')


class AdminOfferCreateView(LoginRequiredMixin, PassRequestToFormKwargsMixin, CreateView):
    '''
    Creates a Product to be added to offers
    '''
    template_name = "vendor/manage/offer.html"
    model = Offer
    form_class = OfferForm

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        context['formset'] = PriceFormSet()

        return context

    def post(self, request):
        offer_form = self.form_class(request.POST, request=request)
        price_formset = PriceFormSet(request.POST)

        if not offer_form.is_valid():
            return render(request, self.template_name, {'form': offer_form, 'formset': price_formset})

        offer = offer_form.save(commit=False)
        if len(offer_form.cleaned_data['products']) > 1:
            offer.bundle = True

        offer.save()
        for product in offer_form.cleaned_data['products']:
            offer.products.add(product)

        if price_formset.has_changed() and not price_formset.is_valid():
            return render(request, self.template_name, {'form': offer_form, 'formset': price_formset})
        elif price_formset.has_changed() and price_formset.is_valid():
            product_currencies = {}
            # price_currency = [price_form.cleaned_data['currency']
            #                   for price_form in price_formset]

            for product in Product.objects.filter(pk__in=offer_form.cleaned_data['products']):
                for currency in product.meta['msrp'].keys():
                    product_currencies[currency] = currency

            for price_form in price_formset:
                if price_form.cleaned_data['currency'] not in product_currencies:
                    price_formset[0].add_error(
                        'currency', _('Invalid currency'))
                    return render(request, self.template_name, {'form': offer_form, 'formset': price_formset})

            for price_form in price_formset:
                price = price_form.save(commit=False)
                if price_form.cleaned_data['price_select'] == 'free':
                    price.cost = 0
                price.offer = offer
                price.save()

        return redirect('vendor_admin:manager-offer-list')


class AdminSubscriptionListView(LoginRequiredMixin, ListView):
    '''
    List of all the invoices generated on the current site.
    '''
    template_name = "vendor/manage/receipt_list.html"
    model = Receipt

    def get_queryset(self):
        queryset = super().get_queryset()
        return queryset.filter(products__in=Product.on_site.all(), order_item__offer__terms__lt=TermType.PERPETUAL)


class AdminSubscriptionDetailView(LoginRequiredMixin, DetailView):
    '''
    Gets all Customer Profile information for quick lookup and management
    '''
    template_name = 'vendor/manage/subscription_detail.html'
    model = Receipt
    slug_field = 'uuid'
    slug_url_kwarg = 'uuid'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        payment = Payment.objects.get(transaction=context['object'].transaction)

        context['payment'] = payment
        context['payment_form'] = CreditCardForm(
            initial={'payment_type': PaymentTypes.CREDIT_CARD})
        context['billing_form'] = AddressForm(instance=payment.billing_address)

        return context


class AdminProfileListView(LoginRequiredMixin, TableFilterMixin, SiteOnRequestFilterMixin, ListView):
    """
    List of CustomerProfiles on site
    """
    template_name = "vendor/manage/profile_list.html"
    model = CustomerProfile
    paginate_by = 100


    def search_filter(self, queryset):
        search_value = self.request.GET.get('search_filter')
        return queryset.filter(Q(pk__icontains=search_value) | \
                               Q(user__email__icontains=search_value))

    def get_paginated_by(self, queryset):
        if 'paginate_by' in self.request.kwargs:
            return kwargs['paginate_by']
        return self.paginate_by
    
    def get_queryset(self):
        queryset = super().get_queryset()

        return queryset.order_by('pk')


class AdminProfileDetailView(LoginRequiredMixin, DetailView):
    '''
    Gets all Customer Profile information for quick lookup and management
    '''
    template_name = 'vendor/manage/profile_detail.html'
    model = CustomerProfile
    slug_field = 'uuid'
    slug_url_kwarg = 'uuid'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        context['free_offers'] = Offer.objects.filter(prices__cost=0, site=self.object.site)

        return context


class AdminManualSubscriptionRenewal(LoginRequiredMixin, DetailView):
    success_url = reverse_lazy('vendor_admin:manage-profiles')
    model = Receipt
    slug_field = 'uuid'
    slug_url_kwarg = 'uuid'

    def post(self, request, *args, **kwargs):
        past_receipt = Receipt.objects.get(uuid=self.kwargs["uuid"])

        payment_info = {
            'msg': 'renewed manually'
        }

        invoice = Invoice(status=Invoice.InvoiceStatus.PROCESSING, site=past_receipt.order_item.invoice.site)
        invoice.profile = past_receipt.profile
        invoice.save()
        invoice.add_offer(past_receipt.order_item.offer)

        processor = get_site_payment_processor(invoice.site)(invoice.site, invoice)
        processor.renew_subscription(past_receipt, payment_info)

        messages.info(request, _("Subscription Renewed"))
        return redirect(request.META.get('HTTP_REFERER', self.success_url))


class PaymentWithNoReceiptListView(LoginRequiredMixin, ListView):
    template_name = "vendor/manage/payment_list.html"
    model = Payment

    def get_queryset(self):
        site = get_site_from_request(self.request)
        return [payment for payment in Payment.objects.filter(invoice__site=site, success=True) if payment.get_receipt() is None]

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = _("Payments with no Receipts")
        return context


class PaymentWithNoOrderItemsListView(LoginRequiredMixin, ListView):
    template_name = "vendor/manage/payment_list.html"
    model = Payment

    def get_queryset(self):
        site = get_site_from_request(self.request)
        return Payment.objects.filter(invoice__site=site, success=True).annotate(order_item_count=Count('invoice__order_items')).filter(order_item_count=0)

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(**kwargs)
        context['title'] = _("Payments with no Order Items")
        return context


class PaymentProcessorSiteConfigsListView(ListView):
    template_name = 'vendor/manage/processor_site_config_list.html'
    model = SiteConfigModel

    def get_queryset(self):
        payment_processor = PaymentProcessorSiteConfig()
        return SiteConfigModel.objects.filter(key=payment_processor.key)


class PaymentProcessorFormView(FormView):
    template_name = 'vendor/manage/processor_site_config.html'
    form_class = PaymentProcessorForm

    def get_success_url(self):
        return reverse('vendor_admin:vendor-processor')

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        processor_config = PaymentProcessorSiteConfig()
        context['form'] = processor_config.get_form()
        return context

    def form_valid(self, form):
        processor_config = PaymentProcessorSiteConfig()
        processor_config.save(form.cleaned_data["payment_processor"], "payment_processor")
        return redirect('vendor_admin:vendor-processor-lists')


class PaymentProcessorSiteSelectFormView(FormView):
    template_name = 'vendor/manage/processor_site_config.html'
    form_class = PaymentProcessorSiteSelectForm

    def get_success_url(self):
        return reverse('vendor_admin:vendor-processor')

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        processor_config = PaymentProcessorSiteSelectSiteConfig(Site.objects.get(pk=self.kwargs.get('pk')))
        context['form'] = processor_config.get_form()
        return context

    def form_valid(self, form):
        site = Site.objects.get(pk=form.cleaned_data['site'])
        processor_config = PaymentProcessorSiteSelectSiteConfig(site)
        processor_config.save(form.cleaned_data["payment_processor"], "payment_processor")
        return redirect('vendor_admin:vendor-processor-lists')


class AuthorizeNetIntegrationView(FormView):
    template_name = "vendor/authorizenet_integration.html"
    form_class = AuthorizeNetIntegrationForm
    success_url = reverse_lazy('authorizenet-integration')

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(**kwargs)
        authorizenet_integration = AuthorizeNetIntegration(get_site_from_request(self.request))
        if authorizenet_integration.instance:
            context['form'] = AuthorizeNetIntegrationForm(instance=authorizenet_integration.instance)
        else:
            context['form'] = AuthorizeNetIntegrationForm()
        return context
    
    def form_valid(self, form):
        authorizenet_integration = AuthorizeNetIntegration(get_site_from_request(self.request))
        authorizenet_integration.save(form.cleaned_data)
        return super().form_valid(form)



# Generated by Django 3.1 on 2020-09-12 01:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('vendor', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='address',
            name='name',
            field=models.CharField(blank=True, max_length=80, verbose_name='Address Name'),
        ),
    ]



import uuid

from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.db import models
from django.utils.translation import gettext_lazy as _

from vendor.models.receipt import Receipt
from vendor.models.base import SoftDeleteModelBase
from vendor.utils import get_display_decimal


##########
# PAYMENT
##########
class Payment(SoftDeleteModelBase):
    '''
    Payments
    - Payments are typically from a Credit Card, PayPal or ACH
    - Multiple Payments can be applied to an invoice
    - Gift cards can be used as payments
    - Discounts are Payment credits
    '''
    uuid = models.UUIDField(_("UUID"), editable=False, unique=True, default=uuid.uuid4, null=False, blank=False)
    invoice = models.ForeignKey("vendor.Invoice", verbose_name=_("Invoice"), on_delete=models.CASCADE, related_name="payments")
    created = models.DateTimeField(_("Date Created"), auto_now_add=True)
    transaction = models.CharField(_("Transaction ID"), max_length=50)
    provider = models.CharField(_("Payment Provider"), max_length=30)
    amount = models.FloatField(_("Amount"))
    profile = models.ForeignKey("vendor.CustomerProfile", verbose_name=_("Purchase Profile"), blank=True, on_delete=models.CASCADE, related_name="payments")
    billing_address = models.ForeignKey("vendor.Address", verbose_name=_("Billing Address"), on_delete=models.CASCADE, blank=True, null=True)
    result = models.JSONField(_("Result"), default=dict, blank=True, null=True)
    success = models.BooleanField(_("Successful"), default=False)
    payee_full_name = models.CharField(_("Name on Card"), max_length=50)
    payee_company = models.CharField(_("Company"), max_length=50, blank=True, null=True)

    def get_related_receipts(self):
        return Receipt.objects.filter(transaction=self.transaction)

    def get_receipt(self):
        try:
            return Receipt.objects.get(transaction=self.transaction, created__year=self.created.year, created__month=self.created.month, created__day=self.created.day)
        except ObjectDoesNotExist:
            return None
        except MultipleObjectsReturned:
            return Receipt.objects.filter(transaction=self.transaction, created__year=self.created.year, created__month=self.created.month, created__day=self.created.day).first()

    def get_amount_display(self):
        return get_display_decimal(self.amount)




from .test_processor import *







# TODO: Check which tests are still worth keepting
# import os
# import json
# import unittest

# from django.test import TestCase, Client, RequestFactory
# from django.urls import reverse
# from django.utils import timezone
# from django.contrib.auth.models import User

# from vendor.models import Offer, Price, Invoice, OrderItem, Refund
# from vendor.models import Product


# ########################
# # API VIEW CLIENT TESTS
# ########################

# ####################
# # CART CLIENT TESTS
# ####################

# ##############
# # ADD TO CART
# ##############

# class AddToCartClientTest(TestCase):
#     '''
#     Tests for AddToCart Functionality
#     '''

#     fixtures = ['unittest']

#     def setUp(self):
#         self.client = Client()
#         self.user = User.objects.get(pk=2)
#         self.product = Product.objects.get(pk=4)

#     def test_add_to_cart_create_invoice(self):
#         '''
#         Test for adding item to the cart when there is no invoice for the user with cart status.
#         '''
#         self.client.force_login(self.user)

#         offer = Offer.objects.create(product = self.product)
#         invoice = Invoice.objects.filter(user = self.user, status = 0).count()

#         data = {
#             "offer": offer.sku
#         }

#         uri = reverse('vendor-add-to-cart-api')
#         response = self.client.post(uri, data)

#         new_invoice = Invoice.objects.filter(user = self.user, status = 0).count()
#         orderitem = OrderItem.objects.filter(offer = offer).count()

#         try:
#             self.assertEqual(response.status_code, 200)     # 200 -> Created Response Code
#             self.assertEqual(invoice, 0)
#             self.assertEqual(new_invoice, 1)
#             self.assertEqual(orderitem, 1)

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response.data)
#             raise


#     def test_add_to_cart_create_order_item(self):
#         '''
#         Test for adding item to the cart
#         '''
#         self.client.force_login(self.user)
#         offer = Offer.objects.create(product = self.product)

#         # offer = Offer.objects.create(product = self.product)
#         invoice = Invoice.objects.create(user = self.user, ordered_date = timezone.now())
#         orderitem = OrderItem.objects.filter(offer = offer, invoice = invoice).count()

#         data = {
#             "offer": offer.sku
#         }

#         uri = reverse('vendor-add-to-cart-api')
#         response = self.client.post(uri, data)

#         new_orderitem = OrderItem.objects.filter(offer = offer, invoice = invoice).count()

#         try:
#             self.assertEqual(response.status_code, 200)     # 200 -> Created Response Code
#             self.assertEqual(orderitem, 0)
#             self.assertEqual(new_orderitem, 1)

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response.data)
#             raise


#     def test_add_to_cart_increment_offer_quantity(self):
#         '''
#         Test for adding item to the cart which is already in the cart
#         '''

#         self.client.force_login(self.user)
#         offer = Offer.objects.get(pk=2)
#         invoice = Invoice.objects.create(user=self.user, ordered_date=timezone.now())
#         orderitem = OrderItem.objects.create(invoice=invoice, offer=offer)

#         data = {
#             "offer": offer.sku
#         }

#         uri = reverse('vendor-add-to-cart-api')             # Add the offer to the current user's cart
#         response = self.client.post(uri, data)

#         try:
#             self.assertEqual(response.status_code, 200)     # 200 -> Created Response Code
#             self.assertEqual(orderitem.quantity, 1)
#             self.assertGreater(OrderItem.objects.get(offer=offer, invoice=invoice).quantity, 1)

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response.data)
#             raise

# ###################
# # DELETE FROM CART
# ###################

# class RemoveItemFromCartClientTest(TestCase):
#     '''
#     Tests for Cart Functionality
#     '''

#     fixtures = ['unittest']

#     def setUp(self):
#         self.client = Client()
#         self.user = User.objects.get(pk=2)
#         self.product = Product.objects.get(pk=4)

#     def test_remove_item_from_cart(self):
#         '''
#         Test for removing the item from the cart
#         '''

#         self.client.force_login(self.user)
#         offer = Offer.objects.get(pk=2)
#         invoice = Invoice.objects.create(user = self.user, ordered_date = timezone.now())
#         orderitem = OrderItem.objects.create(invoice = invoice, offer = offer)

#         uri = reverse('vendor-remove-from-cart-api', kwargs={'sku': offer.sku})
#         response = self.client.patch(uri)

#         order = OrderItem.objects.filter(offer = offer, invoice = invoice).count()

#         try:
#             self.assertEqual(response.status_code, 200)     # 200 -> Created Response Code
#             self.assertEqual(order, 0)

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response.data)
#             raise


#     def test_remove_item_from_cart_fail_1(self):
#         '''
#         Test for removing the item not present in the cart
#         '''

#         self.client.force_login(self.user)
#         offer = Offer.objects.get(pk=2)
#         invoice = Invoice.objects.create(user = self.user, ordered_date = timezone.now())

#         uri = reverse('vendor-remove-from-cart-api', kwargs={'sku': offer.sku})
#         response = self.client.patch(uri)

#         try:
#             self.assertEqual(response.status_code, 404)     # 404 -> Created Response Code

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response.data)
#             raise


#     def test_remove_item_from_cart_fail_2(self):
#         '''
#         Test for removing the item from the cart with no active cart for the user
#         '''

#         self.client.force_login(self.user)
#         offer = Offer.objects.get(pk=2)

#         uri = reverse('vendor-remove-from-cart-api', kwargs={'sku': offer.sku})
#         response = self.client.patch(uri)

#         try:
#             self.assertEqual(response.status_code, 400)     # 404 -> Created Response Code

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response.data)
#             raise


# ##################################
# # DECREASE ITEM QUANTITY IN CART
# ##################################

# class DecreaseItemQuantityClientTest(TestCase):
#     '''
#     Tests for Cart Functionality
#     '''

#     fixtures = ['unittest']

#     def setUp(self):
#         self.client = Client()
#         self.user = User.objects.get(pk=2)
#         self.product = Product.objects.get(pk=4)

#     def test_decrease_quantity_from_cart(self):
#         '''
#         Test for decreasing the quantity of the item already in the cart
#         '''

#         self.client.force_login(self.user)
#         offer = Offer.objects.get(pk=2)
#         invoice = Invoice.objects.create(user = self.user, ordered_date = timezone.now())
#         orderitem = OrderItem.objects.create(invoice = invoice, offer = offer)
        
#         data = {
#             "offer": offer.sku
#         }

#         increase_item_quantity_uri = reverse('vendor-add-to-cart-api')
#         cart_response = self.client.post(increase_item_quantity_uri, data)

#         quantity = OrderItem.objects.get(offer = offer, invoice = invoice).quantity

#         uri = reverse('vendor-remove-single-item-from-cart-api', kwargs={'sku': offer.sku})
#         response = self.client.patch(uri)

#         try:
#             self.assertEqual(response.status_code, 200)     # 200 -> Created Response Code
#             self.assertEqual(quantity, 2)
#             self.assertLess(OrderItem.objects.get(offer = offer, invoice = invoice).quantity, quantity)

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response.data)
#             raise


#     def test_decrease_quantity_from_cart_fail_1(self):
#         '''
#         Test for decreasing the quantity of the item when there is no active cart for the user
#         '''

#         self.client.force_login(self.user)
#         offer = Offer.objects.get(pk=2)

#         uri = reverse('vendor-remove-single-item-from-cart-api', kwargs={'sku': offer.sku})
#         response = self.client.patch(uri)

#         try:
#             self.assertEqual(response.status_code, 400)     # 400 -> Created Response Code

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response.data)
#             raise


#     def test_decrease_quantity_from_cart_fail_2(self):
#         '''
#         Test for decreasing the quantity of the item not present in the cart
#         '''

#         self.client.force_login(self.user)
#         offer = Offer.objects.get(pk=2)
#         invoice = Invoice.objects.create(user = self.user, ordered_date = timezone.now())

#         uri = reverse('vendor-remove-single-item-from-cart-api', kwargs={'sku': offer.sku})
#         response = self.client.patch(uri)

#         try:
#             self.assertEqual(response.status_code, 404)     # 404 -> Created Response Code

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response.data)
#             raise


# ##################################
# # INCREASE ITEM QUANTITY IN CART
# ##################################

# class IncreaseItemQuantityTest(TestCase):
#     '''
#     Tests for Cart Functionality
#     '''

#     fixtures = ['unittest']

#     def setUp(self):
#         self.client = Client()
#         self.user = User.objects.get(pk=2)
#         self.product = Product.objects.get(pk=4)

#     def test_increase_item_quantity(self):
#         '''
#         Test for increasing the quantity of an item in the cart
#         '''

#         self.client.force_login(self.user)
#         offer = Offer.objects.get(pk=2)
#         invoice = Invoice.objects.create(user = self.user, ordered_date = timezone.now())
#         orderitem = OrderItem.objects.create(invoice = invoice, offer = offer)

#         quantity = OrderItem.objects.get(offer = offer, invoice = invoice).quantity

#         uri = reverse('vendor-increase-item-quantity-api', kwargs={'sku': offer.sku})
#         response = self.client.patch(uri)

#         updated_quantity = OrderItem.objects.get(offer = offer, invoice = invoice).quantity

#         try:
#             self.assertEqual(response.status_code, 200)     # 200 -> Created Response Code
#             self.assertGreater(updated_quantity, quantity)

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response.data)
#             raise

#     def test_increase_item_quantity_fail_1(self):
#         '''
#         Test for increasing the quantity of the item when there is no active cart for the user
#         '''

#         self.client.force_login(self.user)
#         offer = Offer.objects.get(pk=2)

#         uri = reverse('vendor-increase-item-quantity-api', kwargs={'sku': offer.sku})
#         response = self.client.patch(uri)

#         try:
#             self.assertEqual(response.status_code, 400)     # 400 -> Created Response Code

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response.data)
#             raise


#     def test_increase_item_quantity_fail_2(self):
#         '''
#         Test for increasing the quantity of the item not present in the cart
#         '''

#         self.client.force_login(self.user)
#         offer = Offer.objects.get(pk=2)
#         invoice = Invoice.objects.create(user = self.user, ordered_date = timezone.now())

#         uri = reverse('vendor-increase-item-quantity-api', kwargs={'sku': offer.sku})
#         response = self.client.patch(uri)

#         try:
#             self.assertEqual(response.status_code, 404)     # 404 -> Created Response Code

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response.data)
#             raise


# #################
# # RETRIEVE CART
# #################

# class RetrieveCartClientTest(TestCase):
#     '''
#     Tests for Cart Functionality
#     '''

#     fixtures = ['unittest']

#     def setUp(self):
#         self.client = Client()
#         self.user = User.objects.get(pk=2)
#         self.product = Product.objects.get(pk=4)
#         self.product2 = Product.objects.get(pk=3)

#     def test_cart_retrieve(self):
#         '''
#         Test for retrieving a cart for the user
#         '''

#         self.client.force_login(self.user)

#         offer = Offer.objects.get(pk=2)     # todo: use fixtures
#         invoice = Invoice.objects.create(user = self.user, ordered_date = timezone.now())
#         orderitem = OrderItem.objects.create(invoice = invoice, offer = offer)
#         offer2 = Offer.objects.create(product = self.product2, name = self.product2.name, msrp = 90.0)
#         orderitem2 = OrderItem.objects.create(invoice = invoice, offer = offer2, quantity = 2)

#         check_data = {
#               "username": "testuser",
#               "order_items": [
#                 {
#                   "sku": "MWSHDGQN",
#                   "name": "Test Product",
#                   "price": "50.00",
#                   "quantity": 1
#                 },
#                 {
#                   "sku": "MWSHDGQN",
#                   "name": "Test Product2",
#                   "price": "90.00",
#                   "quantity": 1
#                 }
#               ],
#               "item_count": 2
#             }

#         uri = reverse('vendor-user-cart-retrieve-api')
#         response = self.client.get(uri)

#         try:
#             self.assertEqual(response.status_code, 200)     # 200 -> Return Response Code
#             self.assertEqual(response.data.keys(), check_data.keys())
#             self.assertEqual(response.data["order_items"][0].keys(), check_data["order_items"][0].keys())
#             self.assertEqual(response.data["item_count"], invoice.order_items.all().count())

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response)
#             print(response.data)
#             raise


#     def test_cart_retrieve_fail(self):
#         '''
#         Test for retrieving a cart when there is no active cart for the user
#         '''

#         self.client.force_login(self.user)
#         offer = Offer.objects.get(pk=2)

#         uri = reverse('vendor-user-cart-retrieve-api')
#         response = self.client.get(uri)

#         try:
#             self.assertEqual(response.status_code, 404)     # 404 -> Return Response Code

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response)
#             raise


# #########################
# # RETRIEVE ORDER SUMMARY
# #########################

# class RetrieveOrderSummaryClientTest(TestCase):
#     '''
#     Tests for retrieving order summary
#     '''

#     fixtures = ['unittest']

#     def setUp(self):
#         self.client = Client()
#         self.user = User.objects.get(pk=2)
#         self.product = Product.objects.get(pk=4)
#         self.product2 = Product.objects.get(pk=3)

#     def test_order_summary_retrieve(self):
#         '''
#         Test for retrieving order summary for the user
#         '''

#         self.client.force_login(self.user)

#         offer = Offer.objects.get(pk=2)
#         price = offer.sale_price.filter(start_date__lte= timezone.now(), end_date__gte=timezone.now()).order_by('priority').first()

#         invoice = Invoice.objects.create(user = self.user, ordered_date = timezone.now())

#         orderitem = OrderItem.objects.create(invoice = invoice, offer = offer)

#         offer2 = Offer.objects.create(product = self.product2, name = self.product2.name, msrp = 90.0)
#         price2 = offer2.sale_price.filter(start_date__lte= timezone.now(), end_date__gte=timezone.now()).order_by('priority').first()

#         orderitem2 = OrderItem.objects.create(invoice = invoice, offer = offer2, quantity = 2)

#         check_data = {
#               "username": "testuser",
#               "order_items": [
#                 {
#                   "sku": "MWSHDGQN",
#                   "name": "Test Product",
#                   "price": "50.00",
#                   "item_total": "50.00",
#                   "quantity": 1
#                 },
#                 {
#                   "sku": "MWSHDGQN",
#                   "name": "Test Product2",
#                   "price": "90.00",
#                   "item_total": "180.00",
#                   "quantity": 1
#                 }
#               ],
#               "total": "230.00"
#             }

#         uri = reverse('vendor-order-summary-retrieve-api')
#         response = self.client.get(uri)

#         try:
#             self.assertEqual(response.status_code, 200)     # 200 -> Return Response Code
#             self.assertEqual(response.data.keys(), check_data.keys())
#             self.assertEqual(response.data["order_items"][0].keys(), check_data["order_items"][0].keys())

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response)
#             print(response.data)
#             raise


#     def test_order_summary_retrieve_fail(self):
#         '''
#         Test for retrieving order summary for the user when there is no active cart
#         '''

#         self.client.force_login(self.user)
#         offer = Offer.objects.get(pk=2)

#         uri = reverse('vendor-order-summary-retrieve-api')
#         response = self.client.get(uri)

#         try:
#             self.assertEqual(response.status_code, 404)     # 404 -> Return Response Code

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response)
#             raise


# ##############
# # DELETE CART
# ##############

# class DeleteCartClientTest(TestCase):
#     '''
#     Test for deleting a cart
#     '''

#     fixtures = ['unittest']

#     def setUp(self):
#         self.client = Client()
#         self.user = User.objects.get(pk=2)
#         self.product = Product.objects.get(pk=4)
#         self.product2 = Product.objects.get(pk=3)

#     def test_user_cart_delete(self):
#         '''
#         Test for deleting a cart contents.
#         '''

#         self.client.force_login(self.user)
        
#         uri = reverse('vendor-user-cart-delete-api')
#         response = self.client.delete(uri)

#         try:
#             self.assertEqual(response.status_code, 200)     # 200 -> Return Response Code
#             self.assertEqual(Invoice.objects.all().filter(status=0).count(), 2)      # There should be 3 carts
            
#             # Check the cart to see that it's empty


#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response)
#             raise


# ########################
# # PURCHASES CLIENT TEST
# ########################

# #####################
# # RETRIEVE PURCHASES
# #####################

# class RetrievePurchasesClientTest(TestCase):
#     '''
#     Test for retrieve user purchases
#     '''

#     fixtures = ['unittest']

#     def setUp(self):
#         self.client = Client()
#         self.user = User.objects.get(pk=2)
#         self.product = Product.objects.get(pk=4)
#         self.product2 = Product.objects.get(pk=3)

#     def test_user_purchase_retrieve(self):
#         '''
#         Test for retrieving user purchases
#         '''

#         self.client.force_login(self.user)

#         # todo: refactor to come from the fixtures...

#         offer = Offer.objects.get(pk=2)

#         invoice = Invoice.objects.create(user = self.user, ordered_date = timezone.now())
#         orderitem = OrderItem.objects.create(invoice = invoice, offer = offer)

#         offer2 = Offer.objects.get(pk=3)

#         orderitem2 = OrderItem.objects.create(invoice = invoice, offer = offer2, quantity = 2)

#         purchase_1 = Purchase.objects.create(user = self.user, order_item = orderitem, product = self.product)

#         purchase_2 = Purchase.objects.create(user = self.user, order_item = orderitem2, product = self.product2)

#         check_data = [
#             {
#                 "sku": "JOS45RB1",
#                 "name": "Test Product",
#                 "price": "50.00",
#                 "quantity": 1,
#                 "start_date": None,
#                 "end_date": None,
#                 "status": 0
#             },
#             {
#                 "sku": "JOS45RB1",
#                 "name": "Test Product2",
#                 "price": "90.00",
#                 "quantity": 2,
#                 "start_date": None,
#                 "end_date": None,
#                 "status": 0
#             }
#         ]

#         uri = reverse('vendor-purchases-retrieve-api')

#         response = self.client.get(uri)

#         try:
#             self.assertEqual(response.status_code, 200)     # 200 -> Return Response Code
#             self.assertEqual(response.data[0].keys(), check_data[0].keys())

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response.data)
#             raise


# #####################
# # PAYMENT PROCESSING
# #####################

# class PaymentProcessingTest(TestCase):
#     '''
#     Tests for Payment Proccessing
#     '''

#     fixtures = ['unittest']

#     def setUp(self):
#         self.client = Client()
#         self.user = User.objects.get(pk=2)
#         self.product = Product.objects.get(pk=4)

#     def test_payment_process(self):
        
#         self.client.force_login(self.user)
#         offer = Offer.objects.get(pk=2)
#         invoice = Invoice.objects.create(user = self.user, ordered_date = timezone.now())
#         orderitem = OrderItem.objects.create(invoice = invoice, offer = offer)

#         uri = reverse('vendor-payment-processing-api')
#         response = self.client.post(uri)

#         purchases = Purchase.objects.filter(user = self.user)
#         invoice_completed = Invoice.objects.get(id = invoice.id)
        
#         try:
#             self.assertEqual(response.status_code, 200)     # 200 -> Created Response Code
#             self.assertEqual(invoice_completed.status, 20)
#             self.assertEqual(purchases.count(), 1)

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(response.data)
#             raise


# #####################
# # REFUND CLIENT TEST
# #####################

# class RefundClientTest(TestCase):
#     '''
#     Tests for refund endpoints
#     '''

#     fixtures = ['unittest']

#     def setUp(self):
#         self.client = Client()
#         self.user = User.objects.get(pk=2)
#         self.product = Product.objects.get(pk=4)

#     def test_refund_request(self):
#         '''
#         Tests for requesting a refund
#         '''

#         self.client.force_login(self.user)

#         offer = Offer.objects.get(pk=2)
#         invoice = Invoice.objects.create(user = self.user, ordered_date = timezone.now())
#         orderitem = OrderItem.objects.create(invoice = invoice, offer = offer)

#         uri = reverse('vendor-payment-processing-api')
#         response = self.client.post(uri)

#         purchase = Purchase.objects.get(user = self.user, order_item = orderitem)

#         refund_uri = reverse('vendor-refund-requesting-api')

#         data = {
#             "purchase": purchase.id,
#             "reason": "wrong description"
#         }

#         refund_response = self.client.post(refund_uri, data)

#         try:
#             self.assertEqual(refund_response.status_code, 200)     # 200 -> Created Response Code
#             self.assertEqual(Purchase.objects.get(user = self.user, order_item = orderitem).status, 20)

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(refund_response.data)
#             raise


#     def test_refund_issue(self):
#         '''
#         Tests for issuing a refund
#         '''

#         self.client.force_login(self.user)

#         offer = Offer.objects.get(pk=2)
#         invoice = Invoice.objects.create(user = self.user, ordered_date = timezone.now())
#         orderitem = OrderItem.objects.create(invoice = invoice, offer = offer)

#         uri = reverse('vendor-payment-processing-api')
#         response = self.client.post(uri)

#         purchase = Purchase.objects.get(user = self.user, order_item = orderitem)

#         refund_uri = reverse('vendor-refund-requesting-api')

#         data = {
#             "purchase": purchase.id,
#             "reason": "wrong description"
#         }

#         refund_response = self.client.post(refund_uri, data)

#         refund = Refund.objects.get(purchase = purchase)

#         refund_issue_uri = reverse('vendor-refund-issue-api', kwargs={'id': refund.id})

#         refund_issue_response = self.client.patch(refund_issue_uri)

#         try:
#             self.assertEqual(refund_response.status_code, 200)     # 200 -> Created Response Code
#             self.assertEqual(Purchase.objects.get(user = self.user, order_item = orderitem).status, 30)
#             self.assertEqual(Refund.objects.get(purchase = purchase).accepted, True)

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(refund_issue_response.data)
#             raise


# #####################
# #####################
# # MODEL CLIENT TESTS
# #####################
# #####################

# ###################
# # OFFER MODEL TEST
# ###################

# class OfferModelTest(TestCase):
#     '''
#     Test for Offer Model Test
#     '''

#     fixtures = ['unittest']

#     def setUp(self):
#         self.client = Client()
#         self.product = Product.objects.get(pk=4)


#     def test_price_object_created(self):

#         offer = Offer.objects.create(product=self.product, msrp = 80)
#         price_object_count = offer.sale_price.all().count()

#         try:
#             self.assertEqual(price_object_count, 1)

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(offer)
#             raise


#     def test_current_price(self):

#         offer = Offer.objects.create(product=self.product, msrp = 80)
#         price = Price.objects.create(offer = offer, cost = 90, priority=2, start_date = timezone.now(), end_date = timezone.now() + timezone.timedelta(days=1))

#         try:
#             self.assertEqual(offer.current_price(), offer.sale_price.all()[0].cost)

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             print(offer)
#             print(offer.current_price())
#             raise


# ###################
# # PRICE MODEL TEST
# ###################

# class PriceModelTest(TestCase):
#     '''
#     Test for Price Model Test
#     '''

#     # fixtures = ['unittest']

#     def setUp(self):
#         pass


# #####################
# # INVOICE MODEL TEST
# #####################

# class InvoiceModelTest(TestCase):
#     '''
#     Test for Invoice Model Test
#     '''

#     # fixtures = ['unittest']

#     def setUp(self):
#         pass


# #######################
# # ORDERITEM MODEL TEST
# #######################

# class OrderItemModelTest(TestCase):
#     '''
#     Test for OrderItem Model Test
#     '''

#     fixtures = ['unittest']

#     def setUp(self):
#         self.client = Client()
#         self.user = User.objects.get(pk=2)
#         self.product = Product.objects.get(pk=4)

#     def test_order_item_total_retrieve(self):
#         '''
#         Test for retrieving OrderItem total based on the price and  quantity
#         '''

#         offer = Offer.objects.get(pk=2)
#         invoice = Invoice.objects.create(user = self.user, ordered_date = timezone.now())
#         orderitem = OrderItem.objects.create(invoice = invoice, offer = offer)

#         try:
#             self.assertEqual(orderitem.total, (orderitem.price * orderitem.quantity))

#         except:     # Only print results if there is an error, but continue to raise the error for the testing tool
#             print("")
#             # print(orderitem.price.all())
#             print(orderitem.quantity)
#             raise


# #######################
# # PURCHASES MODEL TEST
# #######################

# class PurchasesModelTest(TestCase):
#     '''
#     Test for Purchases Model Test
#     '''

#     # fixtures = ['unittest']

#     def setUp(self):
#         pass



from django.urls import path

from vendor.views import vendor as vendor_views

app_name = "vendor"

urlpatterns = [
    path('cart/', vendor_views.CartView.as_view(), name="cart"),
    path('checkout/summary/<uuid:uuid>/', vendor_views.PaymentSummaryView.as_view(), name="purchase-summary"),

    path('checkout/account/', vendor_views.AccountInformationView.as_view(), name="checkout-account"),
    path('checkout/payment/', vendor_views.PaymentView.as_view(), name="checkout-payment"),
    path('checkout/review/', vendor_views.ReviewCheckoutView.as_view(), name="checkout-review"),

    path('customer/products/', vendor_views.ProductsListView.as_view(), name="customer-products"),
    path('customer/product/<uuid:uuid>/receipt/', vendor_views.ReceiptDetailView.as_view(), name="customer-receipt"),
    path('customer/subscriptions/', vendor_views.SubscriptionsListView.as_view(), name="customer-subscriptions"),
    path('customer/subscription/update/<uuid:uuid>/payment', vendor_views.SubscriptionUpdatePaymentView.as_view(), name="customer-subscription-update-payment"),
    path('customer/shipping/<uuid:uuid>/update', vendor_views.ShippingAddressUpdateView.as_view(), name="customer-shipping-update"),               # TODO: [GK-3030] Do not use PKs in URLs
    path('customer/billing/<uuid:uuid>/update', vendor_views.AddressUpdateView.as_view(), name="customer-billing-update"),

    # TODO: Add user's account mangement urls
    # TODO: add user's order details page
]
